//
// Created by Tinghua Huang on 19/12/30.
//

#include <getopt.h>
#include <map>
#include <string>
#include <iostream>

#include "grit.h"

//#include "gerea.h"
unsigned int cpus = 1;
vector<vector<double> > bgseq_scores;
vector<string> stat_results;
mutex thread_lock;
mutex thread_lock2;
unsigned int finished_task_n = 0;
unsigned int task_n;

char const short_options[] = "hm:i:k:b:x:n:z:s:t:p:c:u:o:d:";
struct option long_options[] =
        {
            {"help", 0, NULL, 'h'},
            {"motif", 1, NULL, 'm'},
            {"homoseq", 1, NULL, 'i'},
            {"min_sn", 1, NULL, 'k'},
            {"background", 1, NULL, 'b'},
            {"species", 1, NULL, 'x'},
            {"sampling_times", 1, NULL, 'n'},
            {"sampling_size", 1, NULL, 'z'},
            {"lratio", 1, NULL, 's'},
            {"pvalue", 1, NULL, 't'},
            {"score", 1, NULL, 'p'},
            {"cous", 1, NULL, 'c'},
            {"seed", 1, NULL, 'u'},
            {"output", 1, NULL, 'o'},
            {"dir", 1, NULL, 'd'},
            {0, 0, 0, 0}
        };

void print_result(string output_file);

int main(int argc, char *argv[]) {

    if (argc == 1)
    {
        printf("grit -m motif -i homoseq -b bgseq -x species [-k min_sn] [-s rs] [-t pvalue] [-p pscore] [-c cpus] [-u seed] -o output\n");
        exit(1);

    }

    string session_name = "my_session";
    unsigned long permutation_n = 1000;
    string mat_file = "motif.txt";
    string bgseq_file = "bgseq.txt";
    string seq_file = "homoseq.txt";
    string output_file = "output.txt";
    unsigned int sampling_size = 200;
    unsigned int sampling_times = 10;
    unsigned int lratio = 2;  //1 for averare lratio, 2 for maximum lratio
    double p_score = 0.0;
    double threshold = 0.05;
    string output_dir = "./";
    int seed_value = -1;
    //unsigned int cpus = 1;
    string species = "";
    unsigned int min_sn = 30;

    int c;
    while((c=getopt_long(argc, argv, short_options, long_options, NULL)) != -1)
    {
        switch (c)
        {
            case 'h':
                printf("grit -m motif -i homoseq -b bgseq -x species [-k min_sn] [-s rs] [-t pvalue] [-p pscore] [-c cpus] [-u seed] -o output\n");
                exit(0);
                break;
            case 'm':
                mat_file = optarg;
                break;
            case 'i':
                seq_file = optarg;
                break;
            case 'k':
                min_sn = atoi(optarg);
                break;
            case 'b':
                bgseq_file = optarg;
                break;
                //cout<<optarg<<endl;
                //target_n = atoi(optarg);
                //break;
            case 'x':
                species = optarg;
                break;
            case 'n':
                sampling_times = atoi(optarg);
                break;
            case 'z':
                sampling_size = atoi(optarg);
                break;
            case 's':
                lratio = atoi(optarg);
                break;
            case 't':
                threshold = atof(optarg);
                break;
            case 'p':
                //cout<<optarg<<endl;
                p_score = atof(optarg);
                break;
            case 'c':
                cpus = atoi(optarg);
                break;
            case 'u':
                seed_value = atoi(optarg);
                break;
            case 'o':
                output_file = optarg;
                break;
            default :
                cout << "invalid option: " << optarg << endl;
                exit(1);
        }
    } 

    raw_score raw_score_instance;
    raw_score_instance.threshold = threshold;
    //raw_score_instance.output_file = output_file;
    raw_score_instance.hit_thresh = p_score;
    raw_score_instance.sampling_times = sampling_times;
    raw_score_instance.sampling_times = sampling_times;
    raw_score_instance.lratio = lratio;
    raw_score_instance.species = species;
    raw_score_instance.min_sn = min_sn;
    //raw_score_instance.permutation_n = permutation_n;
    //raw_score_instance.dir = output_dir;
    //raw_score_instance.session = session_name;
    //raw_score_instance.cpus = cpus;

    raw_score_instance.mat_file = mat_file;
    raw_score_instance.get_motifs();
    task_n = raw_score_instance.ds_motifs.size();
    //cout << "Read motifs finished" << endl;

    raw_score_instance.bgseq_file = bgseq_file;
    //raw_score_instance.get_seqs();
    raw_score_instance.get_bgseqs();
    //raw_score_instance.get_rdm_bsgroups(seed_value);
    //raw_score_instance.get_mutibgbase_probs();
    raw_score_instance.get_bgbase_probs();
    //cout << "Read background sequences finished" << endl;

    raw_score_instance.seq_file = seq_file;
    //raw_score_instance.get_seqs();
    raw_score_instance.get_mutiseqs();
    //raw_score_instance.get_seq_scode();
    //cout << "Read sequences finished" << endl;

    raw_score_instance.get_mutibase_probs();
    //raw_score_instance.analysis_motifs_wks();
    //cout<<"test 03 finished."<<endl;
    //raw_score_instance.analysis_motifs_st();
    performe_threading(&raw_score_instance, cpus);
    print_result(output_file);

    //cout << "test07" << endl;
    //cout << "Analysis sequences finished" << endl;

}

void print_result(string output_file) {
    ofstream *stat_writer_p;
    string stat_file = output_file;
    stat_writer_p = new ofstream(stat_file.c_str(), ios::out );
    //*stat_writer_p.open(stat_file.c_str(), ios::out);
    if (!*stat_writer_p) mcf_die("Sorry, couldn't open " + stat_file);
    *stat_writer_p << "# motif_name\tscode\tgroup_name\tgroup_size\tchrom\tstrand\tstart\tend\tbinding_site\tposition\tscore\tccv\tsd\tpvalue\tfdr" << endl;
    for (unsigned int li = 0; li < stat_results.size(); li++) {
        *stat_writer_p << stat_results[li] << endl;
    }
    (*stat_writer_p).close();
}
